@@ -1,5 +1,5 @@
 /* tc-s390.c -- Assemble for the S390
-   Copyright (C) 2000-2016 Free Software Foundation, Inc.
+   Copyright (C) 2000-2017 Free Software Foundation, Inc.
    Contributed by Martin Schwidefsky (schwidefsky@de.ibm.com).
 
    This file is part of GAS, the GNU Assembler.
@@ -289,7 +289,9 @@ s390_parse_cpu (const char *         arg,
     { STRING_COMMA_LEN ("zEC12"), STRING_COMMA_LEN ("arch10"),
       S390_INSTR_FLAG_HTM },
     { STRING_COMMA_LEN ("z13"), STRING_COMMA_LEN ("arch11"),
-      S390_INSTR_FLAG_HTM | S390_INSTR_FLAG_VX }
+      S390_INSTR_FLAG_HTM | S390_INSTR_FLAG_VX },
+    { STRING_COMMA_LEN ("arch12"), STRING_COMMA_LEN (""),
+      S390_INSTR_FLAG_HTM | S390_INSTR_FLAG_VX | S390_INSTR_FLAG_VX2 }
   };
   static struct
   {
@@ -301,7 +303,9 @@ s390_parse_cpu (const char *         arg,
     { "htm",   S390_INSTR_FLAG_HTM, TRUE },
     { "nohtm", S390_INSTR_FLAG_HTM, FALSE },
     { "vx",    S390_INSTR_FLAG_VX, TRUE },
-    { "novx",  S390_INSTR_FLAG_VX, FALSE }
+    { "novx",  S390_INSTR_FLAG_VX, FALSE },
+    { "vx2",    S390_INSTR_FLAG_VX2, TRUE },
+    { "novx2",  S390_INSTR_FLAG_VX2, FALSE }
   };
   unsigned int icpu;
   char *ilp_bak;
@@ -966,7 +970,7 @@ s390_exp_compare (expressionS *exp1, expressionS *exp2)
     }
 }
 
-/* Test for @lit and if its present make an entry in the literal pool and
+/* Test for @lit and if it's present make an entry in the literal pool and
    modify the current expression to be an offset into the literal pool.  */
 static elf_suffix_type
 s390_lit_suffix (char **str_p, expressionS *exp_p, elf_suffix_type suffix)
@@ -1096,7 +1100,7 @@ s390_lit_suffix (char **str_p, expressionS *exp_p, elf_suffix_type suffix)
     }
 
   /* Now change exp_p to the offset into the literal pool.
-     Thats the expression: .L^Ax^By-.L^Ax   */
+     That's the expression: .L^Ax^By-.L^Ax   */
   exp_p->X_add_symbol = lpe->sym;
   exp_p->X_op_symbol = lp_sym;
   exp_p->X_op = O_subtract;
@@ -1466,7 +1470,7 @@ md_gather_operands (char *str,
 	  if (*str != '(')
 	    {
 	      /* Check if parenthesized block can be skipped. If the next
-		 operand is neiter an optional operand nor a base register
+		 operand is neither an optional operand nor a base register
 		 then we have a syntax error.  */
 	      operand = s390_operands + *(++opindex_ptr);
 	      if (!(operand->flags & (S390_OPERAND_INDEX|S390_OPERAND_BASE)))
@@ -1486,7 +1490,7 @@ md_gather_operands (char *str,
 			  operand = s390_operands + *(++opindex_ptr);
 			  if (operand->flags & S390_OPERAND_OPTIONAL)
 			    continue;
-			  as_bad (_("syntax error; expected ,"));
+			  as_bad (_("syntax error; expected ','"));
 			  break;
 			}
 		    }
@@ -1516,7 +1520,7 @@ md_gather_operands (char *str,
 	}
       else if (operand->flags & S390_OPERAND_BASE)
 	{
-	  /* After the base register the parenthesed block ends.  */
+	  /* After the base register the parenthesised block ends.  */
 	  if (*str++ != ')')
 	    as_bad (_("syntax error; missing ')' after base register"));
 	  skip_optional = 0;
@@ -1530,7 +1534,7 @@ md_gather_operands (char *str,
 		      operand = s390_operands + *(++opindex_ptr);
 		      if (operand->flags & S390_OPERAND_OPTIONAL)
 			continue;
-		      as_bad (_("syntax error; expected ,"));
+		      as_bad (_("syntax error; expected ','"));
 		      break;
 		    }
 		}
@@ -1564,7 +1568,7 @@ md_gather_operands (char *str,
 		      operand = s390_operands + *(++opindex_ptr);
 		      if (operand->flags & S390_OPERAND_OPTIONAL)
 			continue;
-		      as_bad (_("syntax error; expected ,"));
+		      as_bad (_("syntax error; expected ','"));
 		      break;
 		    }
 		}
@@ -1652,6 +1656,9 @@ md_gather_operands (char *str,
 	      || fixups[i].reloc == BFD_RELOC_390_GOT20
 	      || fixups[i].reloc == BFD_RELOC_390_GOT16)
 	    fixP->fx_no_overflow = 1;
+
+	  if (operand->flags & S390_OPERAND_PCREL)
+	    fixP->fx_pcrel_adjust = operand->shift / 8;
 	}
       else
 	fix_new_exp (frag_now, f - frag_now->fr_literal, 4, &fixups[i].exp,
@@ -2306,6 +2313,7 @@ md_apply_fix (fixS *fixP, valueT *valP, segT seg ATTRIBUTE_UNUSED)
 	  fixP->fx_size = 2;
 	  fixP->fx_where += 1;
 	  fixP->fx_offset += 1;
+	  fixP->fx_pcrel_adjust = 1;
 	  fixP->fx_r_type = BFD_RELOC_390_PC12DBL;
 	}
       else if (operand->bits == 16 && operand->shift == 16)
@@ -2316,16 +2324,27 @@ md_apply_fix (fixS *fixP, valueT *valP, segT seg ATTRIBUTE_UNUSED)
 	    {
 	      fixP->fx_r_type = BFD_RELOC_390_PC16DBL;
 	      fixP->fx_offset += 2;
+	      fixP->fx_pcrel_adjust = 2;
 	    }
 	  else
 	    fixP->fx_r_type = BFD_RELOC_16;
 	}
+      else if (operand->bits == 16 && operand->shift == 32
+	       && (operand->flags & S390_OPERAND_PCREL))
+	{
+	  fixP->fx_size = 2;
+	  fixP->fx_where += 4;
+	  fixP->fx_offset += 4;
+	  fixP->fx_pcrel_adjust = 4;
+	  fixP->fx_r_type = BFD_RELOC_390_PC16DBL;
+	}
       else if (operand->bits == 24 && operand->shift == 24
 	       && (operand->flags & S390_OPERAND_PCREL))
 	{
 	  fixP->fx_size = 3;
 	  fixP->fx_where += 3;
 	  fixP->fx_offset += 3;
+	  fixP->fx_pcrel_adjust = 3;
 	  fixP->fx_r_type = BFD_RELOC_390_PC24DBL;
 	}
       else if (operand->bits == 32 && operand->shift == 16
@@ -2334,6 +2353,7 @@ md_apply_fix (fixS *fixP, valueT *valP, segT seg ATTRIBUTE_UNUSED)
 	  fixP->fx_size = 4;
 	  fixP->fx_where += 2;
 	  fixP->fx_offset += 2;
+	  fixP->fx_pcrel_adjust = 2;
 	  fixP->fx_r_type = BFD_RELOC_390_PC32DBL;
 	}
       else
@@ -2369,7 +2389,7 @@ md_apply_fix (fixS *fixP, valueT *valP, segT seg ATTRIBUTE_UNUSED)
 	case BFD_RELOC_390_PC12DBL:
 	case BFD_RELOC_390_PLT12DBL:
 	  if (fixP->fx_pcrel)
-	    value++;
+	    value += fixP->fx_pcrel_adjust;
 
 	  if (fixP->fx_done)
 	    {
@@ -2420,14 +2440,14 @@ md_apply_fix (fixS *fixP, valueT *valP, segT seg ATTRIBUTE_UNUSED)
 	  break;
 	case BFD_RELOC_390_PC16DBL:
 	case BFD_RELOC_390_PLT16DBL:
-	  value += 2;
+	  value += fixP->fx_pcrel_adjust;
 	  if (fixP->fx_done)
 	    md_number_to_chars (where, (offsetT) value >> 1, 2);
 	  break;
 
 	case BFD_RELOC_390_PC24DBL:
 	case BFD_RELOC_390_PLT24DBL:
-	  value += 3;
+	  value += fixP->fx_pcrel_adjust;
 	  if (fixP->fx_done)
 	    {
 	      unsigned int mop;
@@ -2465,7 +2485,7 @@ md_apply_fix (fixS *fixP, valueT *valP, segT seg ATTRIBUTE_UNUSED)
 	case BFD_RELOC_390_GOTPCDBL:
 	case BFD_RELOC_390_GOTENT:
 	case BFD_RELOC_390_GOTPLTENT:
-	  value += 2;
+	  value += fixP->fx_pcrel_adjust;
 	  if (fixP->fx_done)
 	    md_number_to_chars (where, (offsetT) value >> 1, 4);
 	  break;
